package ij.measure;

/** Calibration objects contain an image's spatial and density calibration data. */
   
public class Calibration {

	public static final int STRAIGHT_LINE=0,POLY2=1,POLY3=2,POLY4=3,
		EXPONENTIAL=4,POWER=5,LOG=6,RODBARD=7;
	public static final int NONE=20, UNCALIBRATED_OD=21;

	/** Pixel width in 'unit's */
	public double pixelWidth = 1.0;
	
	/** Pixel height in 'unit's */
	public double pixelHeight = 1.0;
	
	/** Pixel depth in 'unit's */
	public double pixelDepth = 1.0;

	/** X origin in pixels. */
	public int xOrigin;

	/** Y origin in pixels. */
	public int yOrigin;

	/** Set true to display cartesian coordinates (not yet implemented). */
	public boolean invertY;

	/** Calibration function coefficients */
	private double[] coefficients;
		
	/* Distance unit (e.g. 'cm', 'inch') */
	private String unit = "pixel";
	
	/* Distance units (e.g. 'microns', 'inches') */
	private String units;

	/* Pixel value unit (e.g. 'gray level', 'OD') */
	private String valueUnit = "Gray Value";

	/* Calibration function ID */
	private int function = NONE;

	/* Calibration table */
	private float[] cTable;
	
	private boolean invertedLut;

	/** Constructs a new Calibration object using the default values. */ 
	public Calibration() {
	}
	
	/** Returns true if this image is spatially calibrated. */
	public boolean scaled() {
		return pixelWidth!=1.0 || pixelHeight!=1.0;
	}
	
   	/** Sets the distance unit (e.g. "mm", "inch"). */
 	public void setUnit(String unit) {
 		if (unit==null)
 			this.unit = "pixel";
 		else
 			this.unit = unit;
 		units = null;
 	}
 	
 	/** Returns the distance unit (e.g. "micron", "inch"). */
 	public String getUnit() {
 		return unit;
 	}
 	
	/** Returns the plural form of the distance unit (e.g. "microns", "inches"). */
 	public String getUnits() {
 		if (units==null) {
  			if (unit.equals("pixel"))
 				units = "pixels";
 			else if (unit.equals("micron"))
 				units = "microns";
  			else if (unit.equals("inch"))
 				units = "inches";
			else
 				units = unit;
 		}
 		return units;
 	}
 	
 	/** Convertes an x-coodinate in pixels to physical units (e.g. mm). */
 	public double getX(int x) {
 		return (x-xOrigin)*pixelWidth;
 	}
 	
  	/** Convertes an x-coodinate in pixels to physical units (e.g. mm). */
 	public double getY(int y) {
 		return (y-yOrigin)*pixelHeight;
 	}
 	
  	/** Sets the calibration function,  coefficient table and unit (e.g. "OD"). */
 	public void setFunction(int function, double[] coefficients, String unit) {
 		if (coefficients==null && function>=STRAIGHT_LINE && function<=RODBARD)
 			return;
 		this.function = function;
 		this.coefficients = coefficients;
 		if (unit!=null)
 			valueUnit = unit;
 		cTable = null;
 	}
 	
 	public void setInvertedLut(boolean invertedLut) {
 		this.invertedLut = invertedLut;
 	}
 	
 	/** Returns the density unit. */
 	public String getValueUnit() {
 		return valueUnit;
 	}
 	
 	/** Returns the calibration function coefficients. */
 	public double[] getCoefficients() {
 		return coefficients;
 	}

 	/** Returns true if this image is density calibrated. */
	public boolean calibrated() {
		return function!=NONE;
	}
	
	/** Returns the calibration function ID. */
 	public int getFunction() {
 		return function;
 	}
 	
	/** Returns the calibration table. */
 	public float[] getCTable() {
 		if (cTable==null)
 			makeCTable();
 		return cTable;
 	}
 	
 	void makeCTable() {
 		if (function==UNCALIBRATED_OD) {
 			cTable = new float[256];
			for (int i=0; i<256; i++)
				cTable[i] = (float)od(i);
		} else if (function>=STRAIGHT_LINE && function<=RODBARD && coefficients!=null) {
 			cTable = new float[256];
 			for (int i=0; i<256; i++)
				cTable[i] = (float)CurveFitter.f(function, coefficients, i);
		} else
 			cTable = null;
  	}

	double od(double v) {
		if (invertedLut) {
			if (v==255.0) v = 254.5;
			return 0.434294481*Math.log(255.0/(255.0-v));
		} else {
			if (v==0.0) v = 0.5;
			return 0.434294481*Math.log(255.0/v);
		}
	}
	
  	/** Converts a raw pixel value to a density calibrated value. */
 	public double getCValue(int value) {
		//ij.IJ.write("getCValue: "+this);
		if (function==NONE)
			return value;
		if (cTable==null)
			makeCTable();
 		if (cTable!=null && value>=0 && value<cTable.length)
 			return cTable[value];
 		else
 			return value;
 	}
 	
	/** Returns a clone of this object. */
	public Calibration copy() {
		Calibration copy = new Calibration();
		copy.pixelWidth = pixelWidth;
		copy.pixelHeight = pixelHeight;
		copy.pixelDepth = pixelDepth;
		copy.unit = unit;
		copy.units = units;
		copy.valueUnit = valueUnit;
		copy.function = function;
		copy.coefficients = coefficients;
		copy.cTable = cTable;
		return copy;
	}
	
   public String toString() {
    	return
    		"w=" + pixelWidth
			+ ", h=" + pixelHeight
			+ ", d=" + pixelDepth
			+ ", unit=" + unit
			+ ", f=" + function
 			+ ", nc=" + (coefficients!=null?""+coefficients.length:"null")
 			+ ", table=" + (cTable!=null?""+cTable.length:"null")
			+ ", vunit=" + valueUnit;
   }
}


